#include "gesture_demostration_objects.h"
#include "gesture_demostration_resources.h"

//--------------------- User code ---------------------//
#define BLUE_MASK 0x001Fu
#define GREEN_MASK 0x07E0u
#define RED_MASK 0xF800u
#define GREEN_APROX 0x0020u

#define CL_MIKROE_YELLOW 0xFE66u
#define CL_MIKROE_GRAY 0x3186u

#define COLOR_PALETTE_X1 30
#define COLOR_PALETTE_Y1 218
#define COLOR_PALETTE_Y2 230
#define COLOR_PALETTE_RESOLUTION 256 //32, 64, 128, 256
#define COLOR_RESOLUTION 256

#define COLOR_INCREMENT 4

#define COLOR_CURSOR_Y1 206u
#define COLOR_CURSOR_Y2 214u
#define COLOR_CURSOR_HEIGHT (COLOR_CURSOR_Y2 - COLOR_CURSOR_Y1)
#define COLOR_CURSOR_DIR_LEFT -1
#define COLOR_CURSOR_DIR_RIGHT 1
#define COLOR_CURSOR_DIR_START 0

#define HORIZONTAL_SCROLL_Y1 19u
#define HORIZONTAL_SCROLL_Y2 24u
#define HORIZONTAL_SCROLL_HEIGHT ( HORIZONTAL_SCROLL_Y2 - HORIZONTAL_SCROLL_Y1 )
#define HORIZONTAL_SCROLL_START 115u
#define HORIZONTAL_SCROLL_WIDTH 184u
#define HORIZONTAL_SCROLL_WIDTH_ADJ (( HORIZONTAL_SCROLL_WIDTH - HORIZONTAL_SCROLL_WIDTH % horizontal_tab_num ) / horizontal_tab_num)

#define VERTICAL_SCROLL_X1 103u
#define VERTICAL_SCROLL_X2 109u
#define VERTICAL_SCROLL_WIDTH ( VERTICAL_SCROLL_X2 - VERTICAL_SCROLL_X1 )
#define VERTICAL_SCROLL_START 30u
#define VERTICAL_SCROLL_HEIGHT 169u
#define VERTICAL_SCROLL_HEIGHT_ADJ (( VERTICAL_SCROLL_HEIGHT - VERTICAL_SCROLL_HEIGHT % vertical_tab_num ) / vertical_tab_num)

#define SELECTOR_WIDTH 3u

int horizontal_tab_num = 3;
int vertical_tab_num = 3;

int v_cursor_location;
int h_cursor_location;

int v_cursor_pos;
int h_cursor_pos;

unsigned int fill_color;
int color_counter;

char color_step_resolution = COLOR_RESOLUTION / COLOR_PALETTE_RESOLUTION;

unsigned long RainbowSelector(unsigned int c_counter){
    unsigned int helper_counter, color;
    int i;
    helper_counter = 0;

    for(i = 0; i <= 31; i++){                     //GREEN up
        helper_counter++;
        color = (((i ) << 6) | RED_MASK) | GREEN_APROX;
        if(helper_counter >= c_counter)
            return color;
    }
    for(i = 31; i >= 0; i--){                     //RED down
        helper_counter++;
        color = ((i << 11) | GREEN_MASK);
        if(helper_counter >= c_counter)
            return color;
    }                                             //BLUE up
    for(i = 0; i <= 31; i++){
        helper_counter++;
        color = (i | GREEN_MASK);
        if(helper_counter >= c_counter)
            return color;
    }
    for(i = 31; i >= 0; i-= 2){                   //GREEN down
        helper_counter++;
        color = ((i << 6) | BLUE_MASK) | GREEN_APROX;
        if(helper_counter >= c_counter)
            return color;
    }
    for(i = 0; i <= 31; i++){                     //RED up
        helper_counter++;
        color = ((i << 11) | BLUE_MASK);
        if(helper_counter >= c_counter)
            return color;
    }
    for(i = 31; i >= 0; i--){                     //BLUE down
        helper_counter++;
        color = (i | RED_MASK);
        if(helper_counter >= c_counter)
            return color;
    }
    for(i = 63; i >= 0; i--){                    //ALL down
        helper_counter++;
        color = ((i/2 << 11) | (i/2 << 6) | i/2) | GREEN_APROX;
        if(helper_counter >= c_counter)
            return color;
    }
    return 0;
}

void DrawColorCursor(unsigned int c_counter, int direction){
    int tmp = c_counter  / color_step_resolution + COLOR_PALETTE_X1;
    TFTIntern_Set_Pen(CL_MIKROE_GRAY,20);
    TFTIntern_V_Line(COLOR_CURSOR_Y1, COLOR_CURSOR_Y2, tmp + COLOR_INCREMENT - 10);
    TFTIntern_V_Line(COLOR_CURSOR_Y1, COLOR_CURSOR_Y2, tmp + COLOR_INCREMENT + 10);

    TFTIntern_Set_Pen(CL_MIKROE_YELLOW,1);
    TFTIntern_V_Line(COLOR_CURSOR_Y1, COLOR_CURSOR_Y2, c_counter / color_step_resolution + COLOR_PALETTE_X1 + COLOR_INCREMENT * direction);
    TFTIntern_V_Line(COLOR_CURSOR_Y1 + COLOR_CURSOR_HEIGHT / 4, COLOR_CURSOR_Y2 - COLOR_CURSOR_HEIGHT / 4, tmp + (COLOR_INCREMENT - 1) * direction);
    TFTIntern_V_Line(COLOR_CURSOR_Y1 + COLOR_CURSOR_HEIGHT / 4, COLOR_CURSOR_Y2 - COLOR_CURSOR_HEIGHT / 4, tmp + (COLOR_INCREMENT + 1) * direction);
    TFTIntern_V_Line(COLOR_CURSOR_Y1 + COLOR_CURSOR_HEIGHT / 4, COLOR_CURSOR_Y2 - COLOR_CURSOR_HEIGHT / 2, tmp + (COLOR_INCREMENT - 2) * direction);
    TFTIntern_V_Line(COLOR_CURSOR_Y1 + COLOR_CURSOR_HEIGHT / 4, COLOR_CURSOR_Y2 - COLOR_CURSOR_HEIGHT / 2, tmp + (COLOR_INCREMENT + 2) * direction);
}

void DrawColorPallete(){
    int i,j;
    for(i = 0; i < COLOR_PALETTE_RESOLUTION; i++){
        for(j = COLOR_PALETTE_Y1; j <= COLOR_PALETTE_Y2; j++){
            TFTIntern_Dot(COLOR_PALETTE_X1 + i, j, RainbowSelector(i * color_step_resolution));
        }
    }
}

void DrawHCursor(int hor,unsigned int color) {
    int i;
    for(i = 0 ; i < HORIZONTAL_SCROLL_HEIGHT ; i++)
        TFTIntern_Dot(hor,HORIZONTAL_SCROLL_Y1 + i,color);
    Delay_ms(1);
}

void DrawVerCursor(int vert,unsigned int color) {
    int i;
    for(i = 0 ; i < VERTICAL_SCROLL_WIDTH ; i++)
        TFTIntern_Dot(VERTICAL_SCROLL_X1 + i,vert,color);
    Delay_ms(1);
}

void ClearRect(int x1, int y1, int x2, int y2){
    TFTIntern_Set_Pen(CL_MIKROE_GRAY,0);
    TFTIntern_Set_Brush(1,CL_MIKROE_GRAY,0,0,0,0);
    TFTIntern_Rectangle(x1,y1,x2,y2);
}

void MoveCursorLeft(){
    int i;
    if(h_cursor_location - HORIZONTAL_SCROLL_WIDTH_ADJ >= HORIZONTAL_SCROLL_START - 6){
        for(i = 1; i <= HORIZONTAL_SCROLL_WIDTH_ADJ; i++){
            DrawHCursor(h_cursor_location + HORIZONTAL_SCROLL_WIDTH_ADJ - i + 1,CL_MIKROE_GRAY);
            DrawHCursor(h_cursor_location - i,CL_MIKROE_YELLOW);
        }
        h_cursor_location -= HORIZONTAL_SCROLL_WIDTH_ADJ;
        h_cursor_pos -= 1;
    }
}

void MoveCursorRight(){
    int i;
    if(h_cursor_location + HORIZONTAL_SCROLL_WIDTH_ADJ < HORIZONTAL_SCROLL_START + HORIZONTAL_SCROLL_WIDTH - 6){
        for(i = 1; i <=  HORIZONTAL_SCROLL_WIDTH_ADJ; i++){
            DrawHCursor(h_cursor_location + HORIZONTAL_SCROLL_WIDTH_ADJ + i ,CL_MIKROE_YELLOW);
            DrawHCursor(h_cursor_location + i - 1,CL_MIKROE_GRAY);
        }
        h_cursor_location += HORIZONTAL_SCROLL_WIDTH_ADJ;
        h_cursor_pos += 1;
    }
}

void MoveCursorUp(){
    int i;
    if(v_cursor_location - VERTICAL_SCROLL_HEIGHT_ADJ >= VERTICAL_SCROLL_START - 6){
        for(i = 1; i <= VERTICAL_SCROLL_HEIGHT_ADJ; i++){
            DrawVerCursor(v_cursor_location + VERTICAL_SCROLL_HEIGHT_ADJ - i + 1,CL_MIKROE_GRAY);
            DrawVerCursor(v_cursor_location - i,CL_MIKROE_YELLOW);
        }
        v_cursor_location -= VERTICAL_SCROLL_HEIGHT_ADJ;
        v_cursor_pos -= 1;
    }
}

void MoveCursorDowm(){
    int i;
    if(v_cursor_location + VERTICAL_SCROLL_HEIGHT_ADJ < VERTICAL_SCROLL_START + VERTICAL_SCROLL_HEIGHT - 10){
        for(i = 1; i <= VERTICAL_SCROLL_HEIGHT_ADJ; i++){
            DrawVerCursor(v_cursor_location + i - 1,CL_MIKROE_GRAY);
            DrawVerCursor(v_cursor_location + i + VERTICAL_SCROLL_HEIGHT_ADJ,CL_MIKROE_YELLOW);
        }
        v_cursor_location += VERTICAL_SCROLL_HEIGHT_ADJ;
        v_cursor_pos += 1;
    }
}

void ClearSelectorRect(){
    TFTIntern_Set_Pen(CL_MIKROE_GRAY,SELECTOR_WIDTH);
    TFTIntern_H_Line(h_cursor_location,h_cursor_location+HORIZONTAL_SCROLL_WIDTH_ADJ,v_cursor_location);
    TFTIntern_H_Line(h_cursor_location,h_cursor_location+HORIZONTAL_SCROLL_WIDTH_ADJ,v_cursor_location + VERTICAL_SCROLL_HEIGHT_ADJ);
    TFTIntern_V_Line(v_cursor_location,v_cursor_location + VERTICAL_SCROLL_HEIGHT_ADJ,h_cursor_location);
    TFTIntern_V_Line(v_cursor_location,v_cursor_location + VERTICAL_SCROLL_HEIGHT_ADJ,h_cursor_location+HORIZONTAL_SCROLL_WIDTH_ADJ);
}

void DrawSelectorRect(){
    TFTIntern_Set_Pen(fill_color,SELECTOR_WIDTH);
    TFTIntern_H_Line(h_cursor_location,h_cursor_location+HORIZONTAL_SCROLL_WIDTH_ADJ,v_cursor_location);
    TFTIntern_H_Line(h_cursor_location,h_cursor_location+HORIZONTAL_SCROLL_WIDTH_ADJ,v_cursor_location + VERTICAL_SCROLL_HEIGHT_ADJ);
    TFTIntern_V_Line(v_cursor_location,v_cursor_location + VERTICAL_SCROLL_HEIGHT_ADJ,h_cursor_location);
    TFTIntern_V_Line(v_cursor_location,v_cursor_location + VERTICAL_SCROLL_HEIGHT_ADJ,h_cursor_location+HORIZONTAL_SCROLL_WIDTH_ADJ);
}

//----------------- End of User code ------------------//

// Event Handlers
void ButtonRound1OnClick() {
    DrawScreen(&Screen2);
    TFTIntern_Set_Pen(CL_MIKROE_YELLOW,0);
    TFTIntern_Set_Brush(1,CL_MIKROE_YELLOW,0,0,0,0);
    TFTIntern_Rectangle(VERTICAL_SCROLL_X1,VERTICAL_SCROLL_START,VERTICAL_SCROLL_X2,VERTICAL_SCROLL_START + VERTICAL_SCROLL_HEIGHT_ADJ);
    TFTIntern_Rectangle(HORIZONTAL_SCROLL_START,HORIZONTAL_SCROLL_Y1,HORIZONTAL_SCROLL_START + HORIZONTAL_SCROLL_WIDTH_ADJ,HORIZONTAL_SCROLL_Y2);


    fill_color = 0xF800;
    v_cursor_location = VERTICAL_SCROLL_START;
    h_cursor_location = HORIZONTAL_SCROLL_START;
    v_cursor_pos = 0;
    h_cursor_pos = 0;
    color_counter = 0;

    DrawColorPallete();
    DrawColorCursor(color_counter - COLOR_INCREMENT, COLOR_CURSOR_DIR_RIGHT);
    DrawSelectorRect();
}

void ButtonRound2OnClick() {
    DrawScreen(&Screen1);
}

void Screen2OnSwipeLeft() {
    ClearSelectorRect();
    MoveCursorLeft();
    DrawSelectorRect();
}

void Screen2OnSwipeRight() {
    ClearSelectorRect();
    MoveCursorRight();
    DrawSelectorRect();
}

void Screen2OnSwipeUp() {
    ClearSelectorRect();
    MoveCursorUp();
    DrawSelectorRect();
}
void Screen2OnSwipeDown() {
    ClearSelectorRect();
    MoveCursorDowm();
    DrawSelectorRect();
}

void Screen2OnZoomIn() {
    if(color_counter + COLOR_INCREMENT <= COLOR_RESOLUTION - 1)
    {
        color_counter += COLOR_INCREMENT;
        fill_color = RainbowSelector(color_counter);
        DrawColorCursor(color_counter,COLOR_CURSOR_DIR_RIGHT);
        DrawSelectorRect();
    }
}

void Screen2OnZoomOut() {
    if(color_counter - COLOR_INCREMENT >= 1){
        color_counter -= COLOR_INCREMENT;
        fill_color = RainbowSelector(color_counter);
        DrawColorCursor(color_counter,COLOR_CURSOR_DIR_LEFT);
        DrawSelectorRect();
    }
}

void ButtonRound3OnClick() {
    TFTIntern_Set_Pen(fill_color,3);
    TFTIntern_Set_Brush(1,fill_color,0,0,0,0);
    TFTIntern_Rectangle(h_cursor_location + SELECTOR_WIDTH,v_cursor_location + SELECTOR_WIDTH,
                  h_cursor_location+HORIZONTAL_SCROLL_WIDTH_ADJ - SELECTOR_WIDTH,
                  v_cursor_location + VERTICAL_SCROLL_HEIGHT_ADJ - SELECTOR_WIDTH);
}

void ButtonRound7OnClick() {
    if(horizontal_tab_num < 9){
        horizontal_tab_num+=1;
        IntToStr(horizontal_tab_num,Label3_Caption);
        ClearRect(115,130,135,160);
        DrawLabel(&Label3);
    }
}

void ButtonRound8OnClick() {
    if(horizontal_tab_num > 2){
        horizontal_tab_num-=1;
        IntToStr(horizontal_tab_num,Label3_Caption);
        ClearRect(115,130,135,160);
        DrawLabel(&Label3);
    }
}

void ButtonRound5OnClick() {
    if(vertical_tab_num < 9){
        vertical_tab_num += 1;
        IntToStr(vertical_tab_num,Label4_Caption);
        ClearRect(115,195,135,220);
        DrawLabel(&Label4);
    }
}

void ButtonRound6OnClick() {
    if(vertical_tab_num > 2){
        vertical_tab_num -= 1;
        IntToStr(vertical_tab_num,Label4_Caption);
        ClearRect(115,195,135,220);
        DrawLabel(&Label4);
    }
}